import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest.library.collection.isdict_containingvalue import has_value
from hamcrest_unit_test.matcher_test import MatcherTest

from .quasidict import QuasiDictionary

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsDictContainingValueTest(MatcherTest):
    def testMatchesSingletonDictionaryContainingValue(self):
        dict = {"a": 1}
        self.assert_matches("same single value", has_value(equal_to(1)), dict)

    def testMatchesDictionaryContainingValue(self):
        dict = {"a": 1, "b": 2, "c": 3}
        self.assert_matches("Matches 1", has_value(equal_to(1)), dict)
        self.assert_matches("Matches 3", has_value(equal_to(3)), dict)

    def testProvidesConvenientShortcutForMatchingWithEqualTo(self):
        dict = {"a": 1, "b": 2, "c": 3}
        self.assert_matches("Matches 3", has_value(3), dict)

    def testDoesNotMatchEmptyDictionary(self):
        self.assert_does_not_match("empty", has_value(1), {})

    def testDoesNotMatchDictionaryMissingValue(self):
        dict = {"a": 1, "b": 2, "c": 3}
        self.assert_does_not_match("no matching value", has_value(4), dict)

    def testMatchesAnyConformingDictionary(self):
        self.assert_matches("quasi-dictionary", has_value("1"), QuasiDictionary())
        self.assert_does_not_match("non-dictionary", has_value("1"), object())

    def testHasReadableDescription(self):
        self.assert_description("a dictionary containing value 'a'", has_value("a"))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(has_value(1), {"a": 1})

    def testMismatchDescriptionShowsActualArgument(self):
        self.assert_mismatch_description("was 'bad'", has_value(1), "bad")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", has_value(1), "bad")


if __name__ == "__main__":
    unittest.main()



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest.core.core.isnot import is_not
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsNotTest(MatcherTest):
    def testEvaluatesToTheTheLogicalNegationOfAnotherMatcher(self):
        self.assert_matches("invert mismatch", is_not(equal_to("A")), "B")
        self.assert_does_not_match("invert match", is_not(equal_to("A")), "A")

    def testProvidesConvenientShortcutForNotEqualTo(self):
        self.assert_matches("invert mismatch", is_not("A"), "B")
        self.assert_does_not_match("invert match", is_not("A"), "A")

    def testProvidesConvenientShortcutForNotInstanceOf(self):
        self.assert_matches("invert mismatch", is_not(str), 1)
        self.assert_does_not_match("invert match", is_not(str), "A")

    def testHasAReadableDescription(self):
        self.assert_description("not 'A'", is_not("A"))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(is_not("A"), "B")

    def testMismatchDescriptionShowsActualArgument(self):
        self.assert_mismatch_description("but was 'A'", is_not("A"), "A")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("but was 'A'", is_not("A"), "A")


if __name__ == "__main__":
    unittest.main()



import unittest

from hamcrest.library.text.isequal_ignoring_whitespace import equal_to_ignoring_whitespace
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


matcher = equal_to_ignoring_whitespace("Hello World   how\n are we? ")


class IsEqualIgnoringWhiteSpaceTest(MatcherTest):
    def testPassesIfWordsAreSameButWhitespaceDiffers(self):
        self.assert_matches("less whitespace", matcher, "Hello World how are we?")
        self.assert_matches("more whitespace", matcher, "   Hello World   how are \n\n\twe?")

    def testFailsIfTextOtherThanWhitespaceDiffers(self):
        self.assert_does_not_match("wrong word", matcher, "Hello PLANET how are we?")
        self.assert_does_not_match("incomplete", matcher, "Hello World how are we")

    def testFailsIfWhitespaceIsAddedOrRemovedInMidWord(self):
        self.assert_does_not_match(
            "need whitespace between Hello and World", matcher, "HelloWorld how are we?"
        )
        self.assert_does_not_match(
            "wrong whitespace within World", matcher, "Hello Wo rld how are we?"
        )

    def testMatcherCreationRequiresString(self):
        self.assertRaises(TypeError, equal_to_ignoring_whitespace, 3)

    def testFailsIfMatchingAgainstNonString(self):
        self.assert_does_not_match("non-string", matcher, object())

    def testDescribesItselfAsIgnoringWhiteSpace(self):
        self.assert_description(
            "'foo\\nbar' ignoring whitespace", equal_to_ignoring_whitespace("foo\nbar")
        )

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(equal_to_ignoring_whitespace("foo\nbar"), "foo bar")

    def testMismatchDescription(self):
        self.assert_mismatch_description("was 'bad'", matcher, "bad")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", matcher, "bad")


if __name__ == "__main__":
    unittest.main()



import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest.library.collection.isdict_containingkey import has_key
from hamcrest_unit_test.matcher_test import MatcherTest

from .quasidict import QuasiDictionary

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsDictContainingKeyTest(MatcherTest):
    def testMatchesSingletonDictionaryContainingKey(self):
        dict = {"a": 1}
        self.assert_matches("same single key", has_key(equal_to("a")), dict)

    def testMatchesDictionaryContainingKey(self):
        dict = {"a": 1, "b": 2, "c": 3}
        self.assert_matches("Matches a", has_key(equal_to("a")), dict)
        self.assert_matches("Matches c", has_key(equal_to("c")), dict)

    def testProvidesConvenientShortcutForMatchingWithEqualTo(self):
        dict = {"a": 1, "b": 2, "c": 3}
        self.assert_matches("Matches c", has_key("c"), dict)

    def testDoesNotMatchEmptyDictionary(self):
        self.assert_does_not_match("empty", has_key("foo"), {})

    def testDoesNotMatchDictionaryMissingKey(self):
        dict = {"a": 1, "b": 2, "c": 3}
        self.assert_does_not_match("no matching key", has_key("d"), dict)

    def testMatchesAnyConformingDictionary(self):
        self.assert_matches("quasi-dictionary", has_key(1), QuasiDictionary())
        self.assert_does_not_match("non-dictionary", has_key(1), object())

    def testHasReadableDescription(self):
        self.assert_description("a dictionary containing key 'a'", has_key("a"))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(has_key("a"), {"a": 1})

    def testMismatchDescriptionShowsActualArgument(self):
        self.assert_mismatch_description("was 'bad'", has_key("a"), "bad")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", has_key("a"), "bad")


if __name__ == "__main__":
    unittest.main()



# coding: utf-8
import platform
from unittest.mock import sentinel

import pytest
from hamcrest.core.base_description import BaseDescription
from hamcrest.core.helpers.ismock import MOCKTYPES
from hamcrest.core.selfdescribing import SelfDescribing

__author__ = "Chris Rose"
__copyright__ = "Copyright 2015 hamcrest.org"
__license__ = "BSD, see License.txt"


class Collector(BaseDescription):
    def __init__(self):
        self.appended = []

    def append(self, obj):
        self.appended.append(obj)


class Described(SelfDescribing):
    def describe_to(self, desc):
        desc.append("described")


@pytest.fixture
def desc():
    return Collector()


def test_append_text_delegates(desc):
    desc.append_text(sentinel.Text)
    assert desc.appended == [sentinel.Text]


@pytest.mark.parametrize(
    "described, appended",
    (
        (Described(), "described"),
        ("unicode-py3", "'unicode-py3'"),
        (b"bytes-py3", "<b'bytes-py3'>"),
        pytest.param(
            "\U0001F4A9",
            "'{0}'".format("\U0001F4A9"),
            marks=pytest.mark.skipif(
                platform.python_implementation() == "PyPy",
                reason="Inexplicable failure on PyPy. Not super important, I hope!",
            ),
        ),
    ),
)
def test_append_description_types(desc, described, appended):
    desc.append_description_of(described)
    assert "".join(desc.appended) == appended


@pytest.mark.parametrize("char, rep", (("'", r"'"), ("\n", r"\n"), ("\r", r"\r"), ("\t", r"\t")))
def test_string_in_python_syntax(desc, char, rep):
    desc.append_string_in_python_syntax(char)
    assert "".join(desc.appended) == "'{0}'".format(rep)


@pytest.mark.parametrize("mock", MOCKTYPES)
def test_describe_mock(desc, mock):
    m = mock()
    desc.append_description_of(m)
    assert "".join(desc.appended) == str(m)



import pytest
from hamcrest.library.text.stringcontains import contains_string
from hamcrest_unit_test.matcher_test import (
    assert_description,
    assert_does_not_match,
    assert_matches,
    assert_mismatch_description,
    assert_no_mismatch_description,
)

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


matcher_args = ("EXCERPT",)


@pytest.fixture(scope="module", params=matcher_args)
def matcher(request):
    return contains_string(request.param)


TEST_MATCHING_STRINGS = (
    ("EXCERPTEND",),
    ("STARTEXCERPTEND",),
    ("STARTEXCERPT",),
    ("EXCERPTEXCERPT",),
    ("EXCERPT",),
)

TEST_MISMATCHING_STRINGS = (("whatever",), ("EXCERP",), (object(),))


@pytest.mark.parametrize(["text"], TEST_MATCHING_STRINGS)
def test_evaluates_true_if_argument_contains_substring(text, matcher):
    assert_matches(matcher, text, "assert that %s matches %s" % (text, matcher))


@pytest.mark.parametrize(["text"], TEST_MISMATCHING_STRINGS)
def test_evaluates_false_with_mismatch(text, matcher):
    assert_does_not_match(matcher, text, "%s was not in string %s" % (matcher, text))


def testMatcherCreationRequiresString():
    with pytest.raises(TypeError):
        contains_string(3)


def test_description(matcher):
    assert_description("a string containing 'EXCERPT'", matcher)


def test_successful_match_does_not_have_mismatch_description(matcher):
    assert_no_mismatch_description(matcher, "EXCERPT")


@pytest.mark.parametrize(["text"], TEST_MISMATCHING_STRINGS)
def test_mismatch_description(matcher, text):
    if isinstance(text, str):
        check_str = "'%s'" % text
    else:
        check_str = "%s" % text
    assert_mismatch_description("was %s" % check_str, matcher, text)


if __name__ == "__main__":
    import unittest

    unittest.main()



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest

from hamcrest import greater_than
from hamcrest.library.object.hasproperty import has_properties, has_property
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Chris Rose"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class OnePropertyOldStyle:

    field = "value"
    field2 = "value2"


class ThreePropertiesNewStyle(object):

    field = "value"
    field2 = "value2"
    field3 = "value3"

    def __repr__(self):
        return "ThreePropertiesNewStyle"

    def __str__(self):
        return repr(self)


class OverridingOldStyle:
    def __getattr__(self, name):
        if name == "field":
            return "value"
        if name == "field2":
            return "value2"

        raise AttributeError(name)


class OverridingNewStyleGetAttr(object):
    def __getattr__(self, name):
        if name == "field":
            return "value"
        if name == "field2":
            return "value2"

        raise AttributeError(name)


class OverridingNewStyleGetAttribute(object):
    def __getattribute__(self, name):
        if name == "field":
            return "value"
        if name == "field2":
            return "value2"

        raise AttributeError(name)


class ObjectPropertyMatcher(object):

    match_sets = (
        ("old-style: %s", OnePropertyOldStyle),
        ("new-style: %s", ThreePropertiesNewStyle),
        ("old-style, overriding: %s", OverridingOldStyle),
        ("new-style, using getattr: %s", OverridingNewStyleGetAttr),
        ("new-style, using getattribute: %s", OverridingNewStyleGetAttribute),
    )

    def assert_matches_for_all_types(self, description, matcher):
        for description_fmt, target_class in self.match_sets:
            self.assert_matches(description_fmt % description, matcher, target_class())

    def assert_does_not_match_for_all_types(self, description, matcher):
        for description_fmt, target_class in self.match_sets:
            self.assert_does_not_match(description_fmt % description, matcher, target_class())


class HasPropertyTest(MatcherTest, ObjectPropertyMatcher):
    def testHasPropertyWithoutValueMatcher(self):
        self.assert_matches_for_all_types("has property with name", has_property("field"))

    def testHasPropertyWithoutValueMatcherNegative(self):
        self.assert_does_not_match_for_all_types(
            "has property with name", has_property("not_there")
        )

    def testHasPropertyWithValueMatcher(self):
        self.assert_matches_for_all_types(
            "has property with name and value", has_property("field", "value")
        )

    def testHasPropertyWithValueMatcherNegative(self):
        self.assert_does_not_match_for_all_types(
            "has property with name", has_property("field", "not the value")
        )

    def testDescription(self):
        self.assert_description(
            "an object with a property 'field' matching ANYTHING", has_property("field")
        )
        self.assert_description(
            "an object with a property 'field' matching 'value'", has_property("field", "value")
        )

    def testDescribeMissingProperty(self):
        self.assert_mismatch_description(
            "<ThreePropertiesNewStyle> did not have the 'not_there' property",
            has_property("not_there"),
            ThreePropertiesNewStyle(),
        )

    def testDescribePropertyValueMismatch(self):
        self.assert_mismatch_description(
            "property 'field' was 'value'",
            has_property("field", "another_value"),
            ThreePropertiesNewStyle(),
        )

    def testMismatchDescription(self):
        self.assert_describe_mismatch(
            "<ThreePropertiesNewStyle> did not have the 'not_there' property",
            has_property("not_there"),
            ThreePropertiesNewStyle(),
        )

    def testNoMismatchDescriptionOnMatch(self):
        self.assert_no_mismatch_description(
            has_property("field", "value"), ThreePropertiesNewStyle()
        )


class HasPropertiesTest(MatcherTest, ObjectPropertyMatcher):
    def testMatcherCreationRequiresEvenNumberOfPositionalArguments(self):
        self.assertRaises(ValueError, has_properties, "a", "b", "c")

    def testMatchesUsingSingleDictionaryArgument(self):
        # import pdb; pdb.set_trace()
        self.assert_matches_for_all_types(
            "matches using a single-argument dictionary",
            has_properties({"field": "value", "field2": "value2"}),
        )

    def testMatchesUsingKeywordArguments(self):
        self.assert_matches_for_all_types(
            "matches using a kwarg dict", has_properties(field="value", field2="value2")
        )

    def testMismatchDescription(self):
        self.assert_describe_mismatch(
            "property 'field' was 'value' and property 'field3' was 'value3'",
            has_properties(field="different", field2="value2", field3="alsodifferent"),
            ThreePropertiesNewStyle(),
        )

    def testDescription(self):
        self.assert_description("an object with a property 'a' matching <1>", has_properties(a=1))
        self.assert_description(
            "an object with properties 'a' matching <1> "
            "and 'b' matching a value greater than <2>",
            has_properties(a=1, b=greater_than(2)),
        )


if __name__ == "__main__":
    unittest.main()



import unittest

from hamcrest.library.text.stringendswith import ends_with
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


EXCERPT = "EXCERPT"
matcher = ends_with(EXCERPT)


class StringEndsWithTest(MatcherTest):
    def testEvaluatesToTrueIfArgumentContainsSpecifiedSubstring(self):
        self.assert_does_not_match("excerpt at beginning", matcher, EXCERPT + "END")
        self.assert_matches("excerpt at end", matcher, "START" + EXCERPT)
        self.assert_does_not_match("excerpt in middle", matcher, "START" + EXCERPT + "END")
        self.assert_matches("excerpt repeated", matcher, EXCERPT + EXCERPT)

        self.assert_does_not_match("excerpt not in string", matcher, "whatever")
        self.assert_does_not_match("only part of excerpt is at end of string", matcher, EXCERPT[1:])

    def testEvaluatesToTrueIfArgumentIsEqualToSubstring(self):
        self.assert_matches("excerpt is entire string", matcher, EXCERPT)

    def testMatcherCreationRequiresString(self):
        self.assertRaises(TypeError, ends_with, 3)

    def testFailsIfMatchingAgainstNonString(self):
        self.assert_does_not_match("non-string", matcher, object())

    def testHasAReadableDescription(self):
        self.assert_description("a string ending with 'EXCERPT'", matcher)

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(matcher, EXCERPT)

    def testMismatchDescription(self):
        self.assert_mismatch_description("was 'bad'", matcher, "bad")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", matcher, "bad")


if __name__ == "__main__":
    unittest.main()



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import re
import unittest

from hamcrest.library.text.stringmatches import matches_regexp
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Chris Rose"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"

string_matcher = matches_regexp(r"--[a-z]+--")
compiled_matcher = matches_regexp(re.compile(r"--[a-z]+--"))


class StringMatchesTest(MatcherTest):
    def testMatchesWhenPatternIsFoundAtBeginning(self):
        self.assert_matches("pattern at beginning", string_matcher, "--a-----")

    def testMatchesWhenPatternIsFoundAtEnd(self):
        self.assert_matches("pattern at end", string_matcher, "-----a--")

    def testMatchesWhenPatternIsFoundInMiddle(self):
        self.assert_matches("pattern in the middle", string_matcher, "-----a-----")

    def testMismatchesWhenPatternIsNotPresent(self):
        self.assert_does_not_match("pattern nowhere", string_matcher, "--0--")

    def testMatchesUsingCompiledExpressions(self):
        self.assert_matches("pattern nowhere", compiled_matcher, "--a--")

    def testMismatchesUsingCompiledExpressions(self):
        self.assert_does_not_match("pattern nowhere", compiled_matcher, "--0--")

    def testStringHasAReadableDescription(self):
        self.assert_description("a string matching '--[a-z]+--'", string_matcher)

    def testPatternHasAReadableDescription(self):
        self.assert_description("a string matching '--[a-z]+--'", compiled_matcher)

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(string_matcher, "--a--")

    def testStringMismatchDescription(self):
        self.assert_mismatch_description("was 'bad'", string_matcher, "bad")

    def testCompiledMismatchDescription(self):
        self.assert_mismatch_description("was 'bad'", compiled_matcher, "bad")

    def testStringDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", string_matcher, "bad")

    def testCompiledDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", compiled_matcher, "bad")


if __name__ == "__main__":
    unittest.main()



class GeneratorForm(object):
    def _sequence(self, *objects):
        for i in objects:
            yield i


class SequenceForm(object):
    def _sequence(self, *objects):
        return list(objects)



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest
from datetime import date

from hamcrest.library.number.ordering_comparison import (
    greater_than,
    greater_than_or_equal_to,
    less_than,
    less_than_or_equal_to,
)
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class OrderingComparisonTest(MatcherTest):
    def testComparesObjectsForGreaterThan(self):
        self.assert_matches("match", greater_than(1), 2)
        self.assert_does_not_match("no match", greater_than(1), 1)

    def testComparesObjectsForLessThan(self):
        self.assert_matches("match", less_than(1), 0)
        self.assert_does_not_match("no match", less_than(1), 1)

    def testComparesObjectsForGreaterThanOrEqualTo(self):
        self.assert_matches("match", greater_than_or_equal_to(1), 2)
        self.assert_matches("match", greater_than_or_equal_to(1), 1)
        self.assert_does_not_match("no match", greater_than_or_equal_to(1), 0)

    def testComparesObjectsForLessThanOrEqualTo(self):
        self.assert_matches("match", less_than_or_equal_to(1), 0)
        self.assert_matches("match", less_than_or_equal_to(1), 1)
        self.assert_does_not_match("no match", less_than_or_equal_to(1), 2)

    def testSupportsDifferentTypesOfComparableObjects(self):
        self.assert_matches("strings", greater_than("bb"), "cc")
        self.assert_matches("dates", less_than(date.today()), date.min)

    def testHasAReadableDescription(self):
        self.assert_description("a value greater than <1>", greater_than(1))
        self.assert_description("a value greater than or equal to <1>", greater_than_or_equal_to(1))
        self.assert_description("a value less than <1>", less_than(1))
        self.assert_description("a value less than or equal to <1>", less_than_or_equal_to(1))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(greater_than(1), 2)
        self.assert_no_mismatch_description(less_than(1), 0)
        self.assert_no_mismatch_description(greater_than_or_equal_to(1), 1)
        self.assert_no_mismatch_description(less_than_or_equal_to(1), 1)

    def testMismatchDescription(self):
        self.assert_mismatch_description("was <0>", greater_than(1), 0)
        self.assert_mismatch_description("was <2>", less_than(1), 2)
        self.assert_mismatch_description("was <0>", greater_than_or_equal_to(1), 0)
        self.assert_mismatch_description("was <2>", less_than_or_equal_to(1), 2)

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was <0>", greater_than(1), 0)
        self.assert_describe_mismatch("was <2>", less_than(1), 2)
        self.assert_describe_mismatch("was <0>", greater_than_or_equal_to(1), 0)
        self.assert_describe_mismatch("was <2>", less_than_or_equal_to(1), 2)


if __name__ == "__main__":
    unittest.main()



import unittest

from hamcrest.library.text.isequal_ignoring_case import equal_to_ignoring_case
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


matcher = equal_to_ignoring_case("heLLo")


class IsEqualIgnoringCaseTest(MatcherTest):
    def testIgnoresCaseOfCharsInString(self):
        self.assert_matches("all upper", matcher, "HELLO")
        self.assert_matches("all lower", matcher, "hello")
        self.assert_matches("mixed up", matcher, "HelLo")

        self.assert_does_not_match("no match", matcher, "bye")

    def testFailsIfAdditionalWhitespaceIsPresent(self):
        self.assert_does_not_match("whitespace suffix", matcher, "heLLo ")
        self.assert_does_not_match("whitespace prefix", matcher, " heLLo")

    def testMatcherCreationRequiresString(self):
        self.assertRaises(TypeError, equal_to_ignoring_case, 3)

    def testFailsIfMatchingAgainstNonString(self):
        self.assert_does_not_match("non-string", matcher, object())

    def testHasAReadableDescription(self):
        self.assert_description("'heLLo' ignoring case", matcher)

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(matcher, "hello")

    def testMismatchDescription(self):
        self.assert_mismatch_description("was 'bad'", matcher, "bad")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", matcher, "bad")


if __name__ == "__main__":
    unittest.main()



import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest.library.collection.issequence_onlycontaining import only_contains
from hamcrest.library.number.ordering_comparison import less_than
from hamcrest_unit_test.matcher_test import MatcherTest

from .quasisequence import QuasiSequence
from .sequencemixin import GeneratorForm, SequenceForm

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsSequenceOnlyContainingTestBase(object):
    def testMatchesSingletonList(self):
        self.assert_matches("singleton list", only_contains(equal_to(1)), self._sequence(1))

    def testMatchesAllItemsWithOneMatcher(self):
        self.assert_matches("one matcher", only_contains(less_than(3)), self._sequence(0, 1, 2))

    def testMatchesAllItemsWithMultipleMatchers(self):
        self.assert_matches(
            "multiple matchers",
            only_contains(less_than(3), equal_to(7)),
            self._sequence(0, 7, 1, 2),
        )

    def testProvidesConvenientShortcutForMatchingWithEqualTo(self):
        self.assert_matches(
            "Values automatically wrapped with equal_to",
            only_contains(less_than(3), 7),
            self._sequence(0, 7, 1, 2),
        )

    def testDoesNotMatchListWithMismatchingItem(self):
        self.assert_does_not_match(
            "3 is not less than 3", only_contains(less_than(3)), self._sequence(1, 2, 3)
        )

    def testDoesNotMatchEmptyList(self):
        self.assert_does_not_match("empty", only_contains("foo"), self._sequence())

    def testMatchesAnyConformingSequence(self):
        class ObjectWithLenOnly(object):
            def __len__(self):
                return 20

        self.assert_matches("quasi-sequence", only_contains(less_than(3)), QuasiSequence())
        self.assert_does_not_match("non-sequence", only_contains(1), object())
        self.assert_does_not_match(
            "non-sequence with length", only_contains(1), ObjectWithLenOnly()
        )

    def testHasAReadableDescription(self):
        self.assert_description(
            "a sequence containing items matching (<1> or <2>)", only_contains(1, 2)
        )

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", only_contains(1, 2), "bad")

    def testDescribeMismatchOfNonSequence(self):
        self.assert_describe_mismatch("was <3>", only_contains(1, 2), 3)


class IsConcreteSequenceOnlyContainingTest(
    MatcherTest, IsSequenceOnlyContainingTestBase, SequenceForm
):
    pass


class IsGeneratorSequenceOnlyContainingTest(
    MatcherTest, IsSequenceOnlyContainingTestBase, GeneratorForm
):
    pass


if __name__ == "__main__":
    unittest.main()



import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest.library.collection.issequence_containinginorder import contains, contains_exactly
from hamcrest_unit_test.matcher_test import MatcherTest

from .quasisequence import QuasiSequence
from .sequencemixin import GeneratorForm, SequenceForm

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsSequenceContainingInOrderTestBase(object):
    def testMatchingSingleItemSequence(self):
        self.assert_matches(
            "Single item sequence", contains_exactly(equal_to(1)), self._sequence(1)
        )

    def testMatchingMultipleItemSequence(self):
        self.assert_matches(
            "Multiple item sequence",
            contains_exactly(equal_to(1), equal_to(2), equal_to(3)),
            self._sequence(1, 2, 3),
        )

    def testProvidesConvenientShortcutForMatchingWithEqualTo(self):
        self.assert_matches(
            "Values automatically wrapped with equal_to",
            contains_exactly(1, 2, 3),
            self._sequence(1, 2, 3),
        )

    def testDoesNotMatchWithMoreElementsThanExpected(self):
        self.assert_mismatch_description(
            "Not matched: <4>", contains_exactly(1, 2, 3), self._sequence(1, 2, 3, 4)
        )

    def testDoesNotMatchWithFewerElementsThanExpected(self):
        self.assert_mismatch_description(
            "No item matched: <3>", contains_exactly(1, 2, 3), self._sequence(1, 2)
        )

    def testDoesNotMatchIfSingleItemMismatches(self):
        self.assert_mismatch_description("item 0: was <3>", contains_exactly(4), self._sequence(3))

    def testDoesNotMatchIfOneOfMultipleItemsMismatch(self):
        self.assert_mismatch_description(
            "item 2: was <4>", contains_exactly(1, 2, 3), self._sequence(1, 2, 4)
        )

    def testDoesNotMatchEmptySequence(self):
        self.assert_mismatch_description(
            "No item matched: <4>", contains_exactly(4), self._sequence()
        )

    def testEmptySequenceMatchesEmptySequence(self):
        self.assert_matches("Empty sequence", contains_exactly(), self._sequence())

    def testMatchesAnyConformingSequence(self):
        self.assert_matches("quasi-sequence", contains_exactly(1, 2), QuasiSequence())
        self.assert_does_not_match("non-sequence", contains_exactly(1, 2), object())

    def testHasAReadableDescription(self):
        self.assert_description("a sequence containing [<1>, <2>]", contains_exactly(1, 2))

    def testDescribeMismatch(self):
        self.assert_describe_mismatch(
            "item 1: was <3>", contains_exactly(1, 2), self._sequence(1, 3)
        )

    def testDescribeMismatchOfNonSequence(self):
        self.assert_describe_mismatch("was <3>", contains_exactly(1, 2), 3)

    def testContainsDeprecated(self):
        self.assert_deprecated("deprecated - use contains_exactly(*items)", contains)


class IsConcreteSequenceContainingInOrderTest(
    MatcherTest, IsSequenceContainingInOrderTestBase, SequenceForm
):
    pass


class IsGeneratorSequenceContainingInOrderTest(
    MatcherTest, IsSequenceContainingInOrderTestBase, GeneratorForm
):
    pass


if __name__ == "__main__":
    unittest.main()



import unittest

from hamcrest.core.core.is_ import is_
from hamcrest.core.core.isequal import equal_to
from hamcrest_unit_test.matcher_test import MatcherTest

from .nevermatch import NeverMatch

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsTest(MatcherTest):
    def testDelegatesMatchingToNestedMatcher(self):
        self.assert_matches("should match", is_(equal_to(True)), True)
        self.assert_matches("should match", is_(equal_to(False)), False)
        self.assert_does_not_match("should not match", is_(equal_to(True)), False)
        self.assert_does_not_match("should not match", is_(equal_to(False)), True)

    def testDescriptionShouldPassThrough(self):
        self.assert_description("<True>", is_(equal_to(True)))

    def testProvidesConvenientShortcutForIsEqualTo(self):
        self.assert_matches("should match", is_("A"), "A")
        self.assert_matches("should match", is_("B"), "B")
        self.assert_does_not_match("should not match", is_("A"), "B")
        self.assert_does_not_match("should not match", is_("B"), "A")
        self.assert_description("'A'", is_("A"))

    def testProvidesConvenientShortcutForIsInstanceOf(self):
        self.assert_matches("should match", is_(str), "A")
        self.assert_does_not_match("should not match", is_(int), "A")

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(is_("A"), "A")

    def testDelegatesMismatchDescriptionToNestedMatcher(self):
        self.assert_mismatch_description(NeverMatch.mismatch_description, is_(NeverMatch()), "hi")

    def testDelegatesDescribeMismatchToNestedMatcher(self):
        self.assert_describe_mismatch(NeverMatch.mismatch_description, is_(NeverMatch()), "hi")


if __name__ == "__main__":
    unittest.main()



import logging
import unittest
import warnings

from hamcrest import anything, assert_that, has_item, has_properties, has_string
from hamcrest.core.string_description import StringDescription

log = logging.getLogger(__name__)

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"
__tracebackhide__ = True


class MatcherTest(unittest.TestCase):
    def assert_matches(self, message, matcher, arg):
        assert_matches(matcher, arg, message)

    def assert_does_not_match(self, message, matcher, arg):
        assert_does_not_match(matcher, arg, message)

    def assert_description(self, expected, matcher):
        assert_description(expected, matcher)

    def assert_no_mismatch_description(self, matcher, arg):
        assert_no_mismatch_description(matcher, arg)

    def assert_mismatch_description(self, expected, matcher, arg):
        assert_mismatch_description(expected, matcher, arg)

    def assert_describe_mismatch(self, expected, matcher, arg):
        assert_describe_mismatch(expected, matcher, arg)

    def assert_deprecated(self, message, matcher):
        assert_deprecated(message, matcher)


def assert_matches(matcher, arg, message):
    try:
        assert matcher.matches(arg), message
    except AssertionError:
        description = StringDescription()
        matcher.describe_mismatch(arg, description)
        log.error(str(description))
        raise


def assert_does_not_match(matcher, arg, message):
    assert not matcher.matches(arg), message


def assert_description(expected, matcher):
    description = StringDescription()
    description.append_description_of(matcher)
    assert expected == str(description)


def assert_no_mismatch_description(matcher, arg):
    description = StringDescription()
    result = matcher.matches(arg, description)
    assert result, "Precondition: Matcher should match item"
    assert "" == str(description), "Expected no mismatch description"


def assert_mismatch_description(expected, matcher, arg):
    description = StringDescription()
    result = matcher.matches(arg, description)
    assert not result, "Precondition: Matcher should not match item"
    assert expected == str(description)


def assert_match_description(expected, matcher, item):
    result = matcher.matches(item, StringDescription())
    assert result, "Precondition: Matcher should match item"

    description = StringDescription()
    matcher.describe_match(item, description)
    assert expected == str(description)


def assert_describe_mismatch(expected, matcher, arg):
    description = StringDescription()
    matcher.describe_mismatch(arg, description)
    assert expected == str(description)


def assert_deprecated(message, matcher):
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")

        matcher(anything()).matches("", StringDescription())

        assert_that(
            w, has_item(has_properties(category=DeprecationWarning, message=has_string(message)))
        )



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")

import unittest

from hamcrest.core.base_matcher import BaseMatcher
from hamcrest_unit_test.matcher_test import assert_match_description, assert_mismatch_description

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class FailingBaseMatcher(BaseMatcher):
    def describe_to(self, description):
        description.append_text("SOME DESCRIPTION")

    def _matches(self, item):
        return False


class PassingBaseMatcher(BaseMatcher):
    def _matches(self, item):
        return True


class BaseMatcherTest(unittest.TestCase):
    def testStrFunctionShouldDescribeMatcher(self):
        matcher = FailingBaseMatcher()
        self.assertEqual("SOME DESCRIPTION", str(matcher))

    def testMismatchDescriptionShouldDescribeItem(self):
        assert_mismatch_description("was <99>", FailingBaseMatcher(), 99)

    def testMatchDescriptionShouldDescribeItem(self):
        assert_match_description("was <99>", PassingBaseMatcher(), 99)

    def testMatcherReprShouldDescribeMatcher(self):
        assert repr(FailingBaseMatcher()) == "<FailingBaseMatcher(SOME DESCRIPTION)>"

    def testMatcherReprShouldTruncateLongDescription(self):
        class LongDescriptionMatcher(BaseMatcher):
            def describe_to(self, description):
                description.append_text("1234 " * 13)

        assert (
            repr(LongDescriptionMatcher())
            == "<LongDescriptionMatcher(1234 1234 1234 1234 1234 1234 1234 1234 1234 1234 1234...)>"
        )


if __name__ == "__main__":
    unittest.main()



import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest.library.collection.issequence_containinginanyorder import contains_inanyorder
from hamcrest_unit_test.matcher_test import MatcherTest

from .quasisequence import QuasiSequence
from .sequencemixin import GeneratorForm, SequenceForm

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsSequenceContainingInAnyOrderBase(object):
    def testMatchingSingleItemSequence(self):
        self.assert_matches(
            "Single item sequence", contains_inanyorder(equal_to(1)), self._sequence(1)
        )

    def testMatchesSequenceInOrder(self):
        self.assert_matches(
            "In order", contains_inanyorder(equal_to(1), equal_to(2)), self._sequence(1, 2)
        )

    def testMatchesSequenceOutOfOrder(self):
        self.assert_matches(
            "Out of order", contains_inanyorder(equal_to(1), equal_to(2)), self._sequence(2, 1)
        )

    def testProvidesConvenientShortcutForMatchingWithEqualTo(self):
        self.assert_matches(
            "Values automatically wrapped with equal_to",
            contains_inanyorder(1, 2),
            self._sequence(2, 1),
        )

    def testDoesNotMatchEmptySequence(self):
        self.assert_mismatch_description(
            "no item matches: <1>, <2> in []", contains_inanyorder(1, 2), self._sequence()
        )

    def testEmptySequenceMatchesEmptySequence(self):
        self.assert_matches("Empty sequence", contains_inanyorder(), self._sequence())

    def testDoesNotMatchIfOneOfMultipleItemsMismatch(self):
        self.assert_mismatch_description(
            "not matched: <4>", contains_inanyorder(1, 2, 3), self._sequence(1, 2, 4)
        )

    def testDoesNotMatchWithMoreElementsThanExpected(self):
        self.assert_mismatch_description(
            "not matched: <2>", contains_inanyorder(1, 3), self._sequence(1, 2, 3)
        )

    def testDoesNotMatchWithFewerElementsThanExpected(self):
        self.assert_mismatch_description(
            "no item matches: <4> in [<1>, <2>, <3>]",
            contains_inanyorder(1, 2, 3, 4),
            self._sequence(1, 2, 3),
        )

    def testMatchesAnyConformingSequence(self):
        self.assert_matches("quasi-sequence", contains_inanyorder(1, 2), QuasiSequence())
        self.assert_does_not_match("non-sequence", contains_inanyorder(1, 2), object())

    def testHasAReadableDescription(self):
        self.assert_description(
            "a sequence over [<1>, <2>] in any order", contains_inanyorder(1, 2)
        )

    def testDescribeMismatch(self):
        self.assert_describe_mismatch(
            "not matched: <3>", contains_inanyorder(1, 2), self._sequence(1, 3)
        )

    def testDescribeMismatchOfNonSequence(self):
        self.assert_describe_mismatch("was <3>", contains_inanyorder(1, 2), 3)

    def testDescribeMismatchAfterMatch(self):
        matcher = contains_inanyorder(1, 2, 3)
        matcher.matches(self._sequence(3, 1))
        self.assert_describe_mismatch(
            "no item matches: <2> in [<3>, <1>]", matcher, self._sequence(3, 1)
        )


class IsConcreteSequenceContainingInAnyOrderTest(
    MatcherTest, IsSequenceContainingInAnyOrderBase, SequenceForm
):
    pass


class IsGeneratorSequenceContainingInAnyOrderTest(
    MatcherTest, IsSequenceContainingInAnyOrderBase, GeneratorForm
):
    pass


if __name__ == "__main__":
    unittest.main()



from collections.abc import Iterator

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class QuasiSequence(object):
    def __iter__(self):
        return QuasiSequenceIterator()

    def __len__(self):
        return 2


class QuasiSequenceIterator(Iterator):
    def __init__(self):
        self.index = 1

    def __iter__(self):
        return self

    def __next__(self):
        if self.index >= 3:
            raise StopIteration
        result = self.index
        self.index += 1
        return result



from collections.abc import Iterator

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class QuasiDictionary(object):
    def items(self):
        return QuasiDictionaryItemIterator()

    def keys(self):
        return QuasiDictionaryKeyIterator()

    def values(self):
        return QuasiDictionaryValueIterator()


class BaseQuasiDictionaryIterator(Iterator):
    def __init__(self):
        self.index = 1

    def __iter__(self):
        return self

    def __next__(self):
        if self.index >= 3:
            raise StopIteration
        result = self.indexToResult()
        self.index += 1
        return result


class QuasiDictionaryItemIterator(BaseQuasiDictionaryIterator):
    def indexToResult(self):
        return (self.index, str(self.index))


class QuasiDictionaryKeyIterator(BaseQuasiDictionaryIterator):
    def indexToResult(self):
        return self.index


class QuasiDictionaryValueIterator(BaseQuasiDictionaryIterator):
    def indexToResult(self):
        return str(self.index)



from hamcrest.library.collection.is_empty import empty
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Chris Rose"
__copyright__ = "Copyright 2012 hamcrest.org"
__license__ = "BSD, see License.txt"


class LengthHaver(object):
    def __init__(self, len_):
        self._len = len_

    def __len__(self):
        return self._len


class EmptyCollectionTest(MatcherTest):
    def testReturnsTrueForEmptyStandardCollections(self):
        matcher = empty()
        self.assert_matches("empty tuple", matcher, ())
        self.assert_matches("empty list", matcher, [])
        self.assert_matches("emtpy dictionary", matcher, {})

    def testReturnsTrueForEmptyCollectionLike(self):
        matcher = empty()
        self.assert_matches("empty protocol object", matcher, LengthHaver(0))

    def testReturnsFalseForNonEmptyStandardCollections(self):
        matcher = empty()
        self.assert_does_not_match("non-empty tuple", matcher, (1,))
        self.assert_does_not_match("non-empty list", matcher, [1])
        self.assert_does_not_match("emtpy dictionary", matcher, {1: 2})

    def testReturnsFalseForNonEmptyCollectionLike(self):
        matcher = empty()
        self.assert_does_not_match("non-empty protocol object", matcher, LengthHaver(1))

    def testHasReadableDescription(self):
        self.assert_description("an empty collection", empty())

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(empty(), [])

    def testDescribeMismatch(self):
        self.assert_mismatch_description("has 3 item(s)", empty(), [1, 2, 3])
        self.assert_mismatch_description("does not support length", empty(), 1)



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsEqualTest(MatcherTest):
    def testComparesObjectsUsingEquality(self):
        self.assert_matches("equal numbers", equal_to(1), 1)
        self.assert_does_not_match("unequal numbers", equal_to(1), 2)

    def testCanCompareNoneValues(self):
        self.assert_matches("None equals None", equal_to(None), None)

        self.assert_does_not_match("None as argument", equal_to("hi"), None)
        self.assert_does_not_match("None in equal_to", equal_to(None), "hi")

    def testHonorsArgumentEqImplementationEvenWithNone(self):
        class AlwaysEqual:
            def __eq__(self, obj):
                return True

        class NeverEqual:
            def __eq__(self, obj):
                return False

        self.assert_matches("always equal", equal_to(None), AlwaysEqual())
        self.assert_does_not_match("never equal", equal_to(None), NeverEqual())

    def testIncludesTheResultOfCallingToStringOnItsArgumentInTheDescription(self):
        argument_description = "ARGUMENT DESCRIPTION"

        class Argument:
            def __str__(self):
                return argument_description

        self.assert_description("<ARGUMENT DESCRIPTION>", equal_to(Argument()))

    def testReturnsAnObviousDescriptionIfCreatedWithANestedMatcherByMistake(self):
        inner_matcher = equal_to("NestedMatcher")
        self.assert_description("<'NestedMatcher'>", equal_to(inner_matcher))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(equal_to("hi"), "hi")

    def testMismatchDescriptionShowsActualArgument(self):
        self.assert_mismatch_description("was 'bad'", equal_to("good"), "bad")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", equal_to("good"), "bad")

    def testEqualToWithEqualBytes(self):
        self.assert_matches("equal for b", equal_to("a"), "a")

    def testNotEqualToWithEqualBytes(self):
        self.assert_does_not_match("equal for b", equal_to("a"), "b")

    def testByteInequalityDescription(self):
        self.assert_mismatch_description("was <{0!r}>".format(b"b"), equal_to(b"a"), b"b")


if __name__ == "__main__":
    unittest.main()



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import re
import unittest

from hamcrest.core.core.issame import same_instance
from hamcrest.core.string_description import StringDescription
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"

ADDRESS_FORMAT = r"-?0x[0-9a-fA-F]+L?"


class IsSameTest(MatcherTest):
    def testEvaluatesToTrueIfArgumentIsReferenceToASpecifiedObject(self):
        o1 = object()
        o2 = object()

        self.assert_matches("same", same_instance(o1), o1)
        self.assert_does_not_match("different", same_instance(o1), o2)

    def testDescriptionIncludesMemoryAddress(self):
        description = StringDescription()
        expected = re.compile("same instance as " + ADDRESS_FORMAT + " 'abc'")

        description.append_description_of(same_instance("abc"))
        self.assertTrue(expected.match(str(description)))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        o1 = object()
        self.assert_no_mismatch_description(same_instance(o1), o1)

    def testMismatchDescriptionShowsActualArgumentAddress(self):
        matcher = same_instance("foo")
        description = StringDescription()
        expected = re.compile("was " + ADDRESS_FORMAT + " 'hi'")

        result = matcher.matches("hi", description)
        self.assertFalse(result, "Precondition: Matcher should not match item")
        self.assertTrue(expected.match(str(description)))

    def testMismatchDescriptionWithNilShouldNotIncludeAddress(self):
        self.assert_mismatch_description("was <None>", same_instance("foo"), None)

    def testDescribeMismatch(self):
        matcher = same_instance("foo")
        description = StringDescription()
        expected = re.compile("was " + ADDRESS_FORMAT + " 'hi'")

        matcher.describe_mismatch("hi", description)
        expected = re.compile("was " + ADDRESS_FORMAT + " 'hi'")
        self.assertTrue(
            expected.match(str(description)),
            "Expected %s to match %s" % (str(matcher), str(description)),
        )

    def testDescribeMismatchWithNilShouldNotIncludeAddress(self):
        self.assert_describe_mismatch("was <None>", same_instance("foo"), None)


if __name__ == "__main__":
    unittest.main()



import sys
import unittest

import pytest
from hamcrest import has_properties, not_
from hamcrest.core.core.raises import calling, raises
from hamcrest_unit_test.matcher_test import MatcherTest, assert_mismatch_description

if __name__ == "__main__":
    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")


__author__ = "Per Fagrell"
__copyright__ = "Copyright 2013 hamcrest.org"
__license__ = "BSD, see License.txt"


def no_exception(*args, **kwargs):
    return


def raise_exception(*args, **kwargs):
    raise AssertionError(str(args) + str(kwargs))


def raise_baseException(*args, **kwargs):
    raise SystemExit(str(args) + str(kwargs))


def raise_exception_with_properties(**kwargs):
    err = AssertionError("boom")
    for k, v in kwargs.items():
        setattr(err, k, v)
    raise err


class RaisesTest(MatcherTest):
    def testMatchesIfFunctionRaisesTheExactExceptionExpected(self):
        self.assert_matches("Right exception", raises(AssertionError), calling(raise_exception))

    def testDoesNotMatchTypeErrorIfActualIsNotCallable(self):
        self.assert_does_not_match("Not callable", raises(TypeError), 23)

    @pytest.mark.skipif(
        not (3, 0) <= sys.version_info < (3, 7), reason="Message differs between Python versions"
    )
    def testDoesNotMatchIfTheWrongExceptionTypeIsRaisedPy3(self):
        self.assert_does_not_match("Wrong exception", raises(IOError), calling(raise_exception))
        expected_message = (
            "AssertionError('(){}',) of type <class 'AssertionError'> was raised instead"
        )
        self.assert_mismatch_description(
            expected_message, raises(TypeError), calling(raise_exception)
        )

    @pytest.mark.skipif(sys.version_info < (3, 7), reason="Message differs between Python versions")
    def testDoesNotMatchIfTheWrongExceptionTypeIsRaisedPy37(self):
        self.assert_does_not_match("Wrong exception", raises(IOError), calling(raise_exception))
        expected_message = (
            "AssertionError('(){}') of type <class 'AssertionError'> was raised instead"
        )
        self.assert_mismatch_description(
            expected_message, raises(TypeError), calling(raise_exception)
        )

    def testMatchesIfFunctionRaisesASubclassOfTheExpectedException(self):
        self.assert_matches("Subclassed Exception", raises(Exception), calling(raise_exception))

    def testMatchesIfFunctionRaisesASubclassOfTheExpectedBaseException(self):
        self.assert_matches(
            "Subclassed BasedException", raises(BaseException), calling(raise_baseException)
        )

    def testDoesNotMatchIfFunctionDoesNotRaiseException(self):
        self.assert_does_not_match("No exception", raises(ValueError), calling(no_exception))

    def testDoesNotMatchExceptionIfRegularExpressionDoesNotMatch(self):
        self.assert_does_not_match(
            "Bad regex", raises(AssertionError, "Phrase not found"), calling(raise_exception)
        )
        self.assert_mismatch_description(
            '''Correct assertion type raised, but the expected pattern ("Phrase not found") not found. Exception message was: "(){}"''',
            raises(AssertionError, "Phrase not found"),
            calling(raise_exception),
        )

    def testMatchesRegularExpressionToStringifiedException(self):
        self.assert_matches(
            "Regex",
            raises(AssertionError, "(3, 1, 4)"),
            calling(raise_exception).with_args(3, 1, 4),
        )

        self.assert_matches(
            "Regex",
            raises(AssertionError, r"([\d, ]+)"),
            calling(raise_exception).with_args(3, 1, 4),
        )

    def testMachesIfRaisedExceptionMatchesAdditionalMatchers(self):
        self.assert_matches(
            "Properties",
            raises(AssertionError, matching=has_properties(prip="prop")),
            calling(raise_exception_with_properties).with_args(prip="prop"),
        )

    def testDoesNotMatchIfAdditionalMatchersDoesNotMatch(self):
        self.assert_does_not_match(
            "Bad properties",
            raises(AssertionError, matching=has_properties(prop="prip")),
            calling(raise_exception_with_properties).with_args(prip="prop"),
        )
        self.assert_mismatch_description(
            '''Correct assertion type raised, but an object with a property 'prop' matching 'prip' not found. Exception message was: "boom"''',
            raises(AssertionError, matching=has_properties(prop="prip")),
            calling(raise_exception_with_properties).with_args(prip="prop"),
        )

    def testDoesNotMatchIfNeitherPatternOrMatcherMatch(self):
        self.assert_does_not_match(
            "Bad pattern and properties",
            raises(AssertionError, pattern="asdf", matching=has_properties(prop="prip")),
            calling(raise_exception_with_properties).with_args(prip="prop"),
        )
        self.assert_mismatch_description(
            '''Correct assertion type raised, but the expected pattern ("asdf") and an object with a property 'prop' matching 'prip' not found. Exception message was: "boom"''',
            raises(AssertionError, pattern="asdf", matching=has_properties(prop="prip")),
            calling(raise_exception_with_properties).with_args(prip="prop"),
        )

    def testDescribeMismatchWillCallItemIfNotTheOriginalMatch(self):
        function = Callable()
        matcher = raises(AssertionError)
        matcher.describe_mismatch(function, object())
        self.assertTrue(function.called)


@pytest.mark.parametrize(
    "expected_message",
    [
        pytest.param(
            "but AssertionError('(){}',) of type <type 'exceptions.AssertionError'> was raised.",
            marks=pytest.mark.skipif(
                sys.version_info >= (3, 0), reason="Message differs between Python versions"
            ),
        ),
        pytest.param(
            "but AssertionError('(){}',) of type <class 'AssertionError'> was raised.",
            marks=pytest.mark.skipif(
                not (3, 0) <= sys.version_info < (3, 7),
                reason="Message differs between Python versions",
            ),
        ),
        pytest.param(
            "but AssertionError('(){}') of type <class 'AssertionError'> was raised.",
            marks=pytest.mark.skipif(
                sys.version_info < (3, 7), reason="Message differs between Python versions"
            ),
        ),
    ],
)
def test_gives_correct_message_when_wrapped_with_is_not(expected_message):
    assert_mismatch_description(
        expected_message, not_(raises(AssertionError)), calling(raise_exception)
    )


class CallingTest(unittest.TestCase):
    def testCallingDoesNotImmediatelyExecuteFunction(self):
        try:
            calling(raise_exception)
        except AssertionError:
            self.fail()
        else:
            pass

    def testCallingObjectCallsProvidedFunction(self):
        method = Callable()
        calling(method)()
        self.assertTrue(method.called)

    def testCallingWithFunctionReturnsObject(self):
        method = Callable()
        callable = calling(method)
        returned = callable.with_args(3, 1, 4, keyword1="arg1")

        self.assertEqual(returned, callable)

    def testCallingWithFunctionSetsArgumentList(self):
        method = Callable()
        calling(method).with_args(3, 1, 4, keyword1="arg1")()

        self.assertEqual(method.args, (3, 1, 4))
        self.assertEqual(method.kwargs, {"keyword1": "arg1"})


class Callable(object):
    def __init__(self):
        self.called = False

    def __call__(self, *args, **kwargs):
        self.called = True
        self.args = args
        self.kwargs = kwargs



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest

from hamcrest.core.core.allof import AllOf, all_of
from hamcrest.core.core.isequal import equal_to
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class AllOfTest(MatcherTest):
    def testMatchesIfArgumentSatisfiesBothOfTwoOtherMatchers(self):
        self.assert_matches("both matchers", all_of(equal_to("good"), equal_to("good")), "good")

    def testProvidesConvenientShortcutForMatchingWithEqualTo(self):
        self.assert_matches("both matchers", all_of("good", "good"), "good")

    def testNoMatchIfArgumentFailsToSatisfyEitherOfTwoOtherMatchers(self):
        self.assert_does_not_match(
            "first matcher", all_of(equal_to("bad"), equal_to("good")), "good"
        )
        self.assert_does_not_match(
            "second matcher", all_of(equal_to("good"), equal_to("bad")), "good"
        )
        self.assert_does_not_match(
            "either matcher", all_of(equal_to("bad"), equal_to("bad")), "good"
        )

    def testMatchesIfArgumentSatisfiesAllOfManyOtherMatchers(self):
        self.assert_matches(
            "all matchers",
            all_of(
                equal_to("good"),
                equal_to("good"),
                equal_to("good"),
                equal_to("good"),
                equal_to("good"),
            ),
            "good",
        )

    def testNoMatchIfArgumentFailsToSatisfyAllOfManyOtherMatchers(self):
        self.assert_does_not_match(
            "matcher in the middle",
            all_of(
                equal_to("good"),
                equal_to("good"),
                equal_to("good"),
                equal_to("bad"),
                equal_to("good"),
                equal_to("good"),
            ),
            "good",
        )

    def testHasAReadableDescription(self):
        self.assert_description(
            "('good' and 'bad' and 'ugly')",
            all_of(equal_to("good"), equal_to("bad"), equal_to("ugly")),
        )

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(all_of(equal_to("good"), equal_to("good")), "good")

    def testMismatchDescriptionDescribesFirstFailingMatch(self):
        self.assert_mismatch_description(
            "'good' was 'bad'", all_of(equal_to("bad"), equal_to("good")), "bad"
        )

    def testDescribeMismatch(self):
        self.assert_describe_mismatch(
            "'good' was 'bad'", all_of(equal_to("bad"), equal_to("good")), "bad"
        )

    def testMismatchDescriptionOptionallyDescribesMultipleFailingMatches(self):
        self.assert_mismatch_description(
            "'bad' was 'indifferent' and 'good' was 'indifferent'",
            AllOf(
                equal_to("bad"),
                equal_to("indifferent"),
                equal_to("good"),
                describe_all_mismatches=True,
            ),
            "indifferent",
        )


if __name__ == "__main__":
    unittest.main()



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest.library.number.ordering_comparison import greater_than
from hamcrest.library.object.haslength import has_length
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class FakeWithLen(object):
    def __init__(self, len):
        self.len = len

    def __len__(self):
        return self.len

    def __str__(self):
        return "FakeWithLen"


class FakeWithoutLen(object):
    def __str__(self):
        return "FakeWithoutLen"


class HasLengthTest(MatcherTest):
    def testPassesResultOfLenToNestedMatcher(self):
        self.assert_matches("equal", has_length(equal_to(42)), FakeWithLen(42))
        self.assert_does_not_match("unequal", has_length(equal_to(42)), FakeWithLen(1))

    def testProvidesConvenientShortcutForHasLengthEqualTo(self):
        self.assert_matches("equal", has_length(42), FakeWithLen(42))
        self.assert_does_not_match("unequal", has_length(42), FakeWithLen(1))

    def testDoesNotMatchObjectWithoutLen(self):
        self.assert_does_not_match("no length", has_length(42), object())

    def testHasReadableDescription(self):
        self.assert_description(
            "an object with length of a value greater than <5>", has_length(greater_than(5))
        )

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(has_length(3), "foo")

    def testMismatchDescriptionForItemWithWrongLength(self):
        self.assert_mismatch_description(
            "was <FakeWithLen> with length of <4>", has_length(3), FakeWithLen(4)
        )

    def testMismatchDescriptionForItemWithoutLength(self):
        self.assert_mismatch_description("was <FakeWithoutLen>", has_length(3), FakeWithoutLen())

    def testDescribeMismatchForItemWithWrongLength(self):
        self.assert_describe_mismatch(
            "was <FakeWithLen> with length of <4>", has_length(3), FakeWithLen(4)
        )

    def testDescribeMismatchForItemWithoutLength(self):
        self.assert_describe_mismatch("was <FakeWithoutLen>", has_length(3), FakeWithoutLen())


if __name__ == "__main__":
    unittest.main()



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest

from hamcrest.core.core.isnone import none, not_none
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsNoneTest(MatcherTest):
    def testEvaluatesToTrueIfArgumentIsNone(self):
        self.assert_matches("None", none(), None)

    def testEvaluatesToFalseIfArgumentIsNotNone(self):
        self.assert_does_not_match("not None", none(), object())

    def testHasAReadableDescription(self):
        self.assert_description("None", none())

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(none(), None)

    def testMismatchDescriptionShowsActualArgument(self):
        self.assert_mismatch_description("was 'bad'", none(), "bad")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", none(), "bad")


class NotNoneTest(MatcherTest):
    def testEvaluatesToTrueIfArgumentIsNotNone(self):
        self.assert_matches("not None", not_none(), object())

    def testEvaluatesToFalseIfArgumentIsNone(self):
        self.assert_does_not_match("None", not_none(), None)

    def testHasAReadableDescription(self):
        self.assert_description("not None", not_none())

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(not_none(), "hi")

    def testMismatchDescriptionShowsActualArgument(self):
        self.assert_mismatch_description("but was <None>", not_none(), None)

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("but was <None>", not_none(), None)


if __name__ == "__main__":
    unittest.main()



from hamcrest.library.text import string_contains_in_order
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Romilly Cocking"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


matcher = string_contains_in_order("string one", "string two", "string three")


class StringContainsInOrderTest(MatcherTest):
    def testMatchesIfOrderIsCorrect(self):
        self.assert_matches(
            "correct order", matcher, "string one then string two followed by string three"
        )

    def testDoesNotMatchIfOrderIsIncorrect(self):
        self.assert_does_not_match(
            "incorrect order", matcher, "string two then string one followed by string three"
        )

    def testDoesNotMatchIfExpectedSubstringsAreMissing(self):
        self.assert_does_not_match("missing string one", matcher, "string two then string three")
        self.assert_does_not_match("missing string two", matcher, "string one then string three")
        self.assert_does_not_match("missing string three", matcher, "string one then string two")

    def testMatcherCreationRequiresString(self):
        self.assertRaises(TypeError, string_contains_in_order, 3)

    def testFailsIfMatchingAgainstNonString(self):
        self.assert_does_not_match("non-string", matcher, object())

    def testHasAReadableDescription(self):
        self.assert_description(
            "a string containing 'string one', 'string two', 'string three' in order", matcher
        )

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(
            matcher, "string one then string two followed by string three"
        )

    def testMismatchDescription(self):
        self.assert_mismatch_description("was 'bad'", matcher, "bad")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", matcher, "bad")



import unittest

from hamcrest.core.core.described_as import described_as
from hamcrest.core.core.isanything import anything
from hamcrest_unit_test.matcher_test import MatcherTest

from .nevermatch import NeverMatch

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class DescribedAsTest(MatcherTest):
    def testOverridesDescriptionOfNestedMatcherWithConstructorArgument(self):
        m1 = described_as("m1 description", anything())
        m2 = described_as("m2 description", NeverMatch())

        self.assert_description("m1 description", m1)
        self.assert_description("m2 description", m2)

    def testAppendsValuesToDescription(self):
        m = described_as("value 1 = %0, value 2 = %1", anything(), 33, 97)

        self.assert_description("value 1 = <33>, value 2 = <97>", m)

    def testDelegatesMatchingToNestedMatcher(self):
        m1 = described_as("irrelevant", anything())
        m2 = described_as("irrelevant", NeverMatch())

        self.assertTrue(m1.matches(object()))
        self.assertTrue(not m2.matches("hi"))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(described_as("irrelevant", anything()), object())

    def testDelegatesMismatchDescriptionToNestedMatcher(self):
        self.assert_mismatch_description(
            NeverMatch.mismatch_description, described_as("irrelevant", NeverMatch()), "hi"
        )

    def testDelegatesDescribeMismatchToNestedMatcher(self):
        self.assert_describe_mismatch(
            NeverMatch.mismatch_description, described_as("irrelevant", NeverMatch()), "hi"
        )


if __name__ == "__main__":
    unittest.main()



import unittest

from hamcrest import starts_with
from hamcrest.core.core.isequal import equal_to
from hamcrest.library.collection.isdict_containing import has_entry
from hamcrest_unit_test.matcher_test import (
    MatcherTest,
    assert_match_description,
    assert_mismatch_description,
)

from .quasidict import QuasiDictionary

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsDictContainingTest(MatcherTest):
    def testMatchesDictionaryContainingMatchingKeyAndValue(self):
        dict = {"a": 1, "b": 2}
        self.assert_matches("has a:1", has_entry(equal_to("a"), equal_to(1)), dict)
        self.assert_matches("has b:2", has_entry(equal_to("b"), equal_to(2)), dict)
        self.assert_does_not_match("no c:3", has_entry(equal_to("c"), equal_to(3)), dict)

    def testProvidesConvenientShortcutForMatchingWithEqualTo(self):
        dict = {"a": 1, "b": 2}
        self.assert_matches("has a:1", has_entry("a", equal_to(1)), dict)
        self.assert_matches("has b:2", has_entry(equal_to("b"), 2), dict)
        self.assert_does_not_match("no c:3", has_entry("c", 3), dict)

    def testMatchesAnyConformingDictionary(self):
        self.assert_matches("quasi-dictionary", has_entry(1, "1"), QuasiDictionary())
        self.assert_does_not_match("non-dictionary", has_entry(1, "1"), object())

    def testHasReadableDescription(self):
        self.assert_description("a dictionary containing ['a': <1>]", has_entry("a", 1))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(has_entry("a", 1), {"a": 1})

    def testMismatchDescriptionShowsActualArgument(self):
        self.assert_mismatch_description("was 'bad'", has_entry("a", 1), "bad")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", has_entry("a", 1), "bad")

    def test_describe_single_matching_key_mismatching_value(self):
        assert_mismatch_description("value for 'a' was <2>", has_entry("a", 1), {"a": 2})
        assert_mismatch_description(
            "value for 'aa' was <2>", has_entry(starts_with("a"), 1), {"aa": 2}
        )
        assert_mismatch_description(
            "was <{'ab': 2, 'ac': 3}>", has_entry(starts_with("a"), 1), {"ab": 2, "ac": 3}
        )

    def test_describe_match(self):
        assert_match_description("value for 'a' was <1>", has_entry("a", 1), {"a": 1, "b": 2})


if __name__ == "__main__":
    unittest.main()



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest.library.collection.isdict_containingentries import has_entries
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsDictContainingEntriesTest(MatcherTest):
    def testMatcherCreationRequiresEvenNumberOfPositionalArgs(self):
        self.assertRaises(ValueError, has_entries, "a", "b", "c")

    def testDoesNotMatchNonDictionary(self):
        self.assert_does_not_match("non-dictionary", has_entries("a", equal_to(1)), object())

    def testMatchesDictLike(self):
        class DictLike(object):
            def __getitem__(self, key):
                return "value: " + str(key)

            def __contains__(self, key):
                return True

        self.assert_matches(
            "matches a dictionary-like object", has_entries("a", equal_to("value: a")), DictLike()
        )

    def testMatchesUsingSingleDictionaryArgument(self):
        target = {"a": 1, "b": 2, "c": 3}
        self.assert_matches("has a & b", has_entries({"a": equal_to(1), "b": equal_to(2)}), target)
        self.assert_matches("has c & a", has_entries({"c": equal_to(3), "a": equal_to(1)}), target)
        self.assert_does_not_match(
            "no d:3", has_entries({"b": equal_to(2), "d": equal_to(3)}), target
        )

    def testMatcheSingleDictionaryArgumentWithImplicitEqualTo(self):
        target = {"a": 1, "b": 2, "c": 3}
        self.assert_matches("has a & b", has_entries({"a": 1, "b": 2}), target)
        self.assert_matches("has c & a", has_entries({"c": 3, "a": 1}), target)
        self.assert_does_not_match("no d:3", has_entries({"b": 2, "d": 3}), target)

    def testMatchesUsingKwargs(self):
        target = {"a": 1, "b": 2, "c": 3}
        self.assert_matches("has a & b", has_entries(a=equal_to(1), b=equal_to(2)), target)
        self.assert_matches("has c & a", has_entries(c=equal_to(3), a=equal_to(1)), target)
        self.assert_does_not_match("no d:3", has_entries(b=equal_to(2), d=equal_to(3)), target)

    def testMatchesKwargsWithImplicitEqualTo(self):
        target = {"a": 1, "b": 2, "c": 3}
        self.assert_matches("has a & b", has_entries(a=1, b=2), target)
        self.assert_matches("has c & a", has_entries(c=3, a=1), target)
        self.assert_does_not_match("no d:3", has_entries(b=2, d=3), target)

    def testMatchesDictionaryContainingSingleKeyWithMatchingValue(self):
        target = {"a": 1, "b": 2}
        self.assert_matches("has a:1", has_entries("a", equal_to(1)), target)
        self.assert_matches("has b:2", has_entries("b", equal_to(2)), target)
        self.assert_does_not_match("no b:3", has_entries("b", equal_to(3)), target)
        self.assert_does_not_match("no c:2", has_entries("c", equal_to(2)), target)

    def testMatchesDictionaryContainingMultipleKeysWithMatchingValues(self):
        target = {"a": 1, "b": 2, "c": 3}
        self.assert_matches("has a & b", has_entries("a", equal_to(1), "b", equal_to(2)), target)
        self.assert_matches("has c & a", has_entries("c", equal_to(3), "a", equal_to(1)), target)
        self.assert_does_not_match(
            "no d:3", has_entries("b", equal_to(3), "d", equal_to(3)), target
        )

    def testProvidesConvenientShortcutForMatchingWithEqualTo(self):
        target = {"a": 1, "b": 2, "c": 3}
        self.assert_matches("has a & b", has_entries("a", 1, "b", 2), target)
        self.assert_matches("has c & a", has_entries("c", 3, "a", 1), target)
        self.assert_does_not_match("no d:4", has_entries("b", 3, "d", 4), target)

    def testHasReadableDescription(self):
        self.assert_description(
            "a dictionary containing {'a': <1>, 'b': <2>}", has_entries("a", 1, "b", 2)
        )

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(has_entries("a", 1), {"a": 1})

    def testMismatchDescriptionOfNonDictionaryShowsActualArgument(self):
        self.assert_mismatch_description(
            "'bad' is not a mapping object", has_entries("a", 1), "bad"
        )

    def testMismatchDescriptionOfDictionaryWithNonMatchingValue(self):
        self.assert_mismatch_description("value for 'a' was <2>", has_entries("a", 1), {"a": 2})

    def testDescribeMismatchOfNonDictionaryShowsActualArgument(self):
        self.assert_describe_mismatch("'bad' is not a mapping object", has_entries("a", 1), "bad")

    def testDescribeMismatchOfDictionaryWithoutKey(self):
        d = {"a": 1, "c": 3}
        self.assert_describe_mismatch("no 'b' key in <%s>" % (d,), has_entries("a", 1, "b", 2), d)

    def testDescribeMismatchOfDictionaryWithNonMatchingValue(self):
        self.assert_describe_mismatch("value for 'a' was <2>", has_entries("a", 1), {"a": 2})


if __name__ == "__main__":
    unittest.main()



import sys

sys.path.append("..")

from hamcrest import *
import unittest


class ExampleWithAssertThat(unittest.TestCase):
    def testUsingAssertThat(self):
        assert_that("xx", is_("xx"))
        assert_that("yy", is_not("xx"))
        assert_that("i like cheese", contains_string("cheese"))

    def testCanAlsoSupplyDescriptiveReason(self):
        assert_that("xx", is_("xx"), "description")

    def testCanAlsoAssertPlainBooleans(self):
        assert_that(True, "This had better not fail")


if __name__ == "__main__":
    unittest.main()



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest.library.object.hasstring import has_string
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class FakeWithStr(object):
    def __str__(self):
        return "FakeWithStr"


class HasStringTest(MatcherTest):
    def testPassesResultOfToStrToNestedMatcher(self):
        self.assert_matches("equal", has_string(equal_to("FakeWithStr")), FakeWithStr())
        self.assert_does_not_match("unequal", has_string(equal_to("FakeWithStr")), 3)

    def testProvidesConvenientShortcutForHasStringEqualTo(self):
        self.assert_matches("equal", has_string("FakeWithStr"), FakeWithStr())
        self.assert_does_not_match("unequal", has_string("FakeWithStr"), 3)

    def testHasReadableDescription(self):
        self.assert_description("an object with str 'foo'", has_string("foo"))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(has_string("FakeWithStr"), FakeWithStr())

    def testMismatchDescription(self):
        self.assert_mismatch_description("was <FakeWithStr>", has_string("foo"), FakeWithStr())

    def testDescribeMismatchDescription(self):
        self.assert_describe_mismatch("was <FakeWithStr>", has_string("foo"), FakeWithStr())


if __name__ == "__main__":
    unittest.main()



import sys
import unittest

from hamcrest.core.core.isinstanceof import instance_of
from hamcrest_unit_test.matcher_test import MatcherTest

if __name__ == "__main__":
    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")


__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsInstanceOfTest(MatcherTest):
    def testEvaluatesToTrueIfArgumentIsInstanceOfASpecificClass(self):
        self.assert_matches("same class", instance_of(int), 1)

        self.assert_does_not_match("different class", instance_of(int), "hi")
        self.assert_does_not_match("None", instance_of(int), None)

    def testMatcherCreationRequiresType(self):
        self.assertRaises(TypeError, instance_of, "not a type")

    def testHasAReadableDescription(self):
        self.assert_description("an instance of int", instance_of(int))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(instance_of(int), 3)

    def testMismatchDescriptionShowsActualArgument(self):
        self.assert_mismatch_description("was 'bad'", instance_of(int), "bad")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", instance_of(int), "bad")


if __name__ == "__main__":
    unittest.main()



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest.library.integration.match_equality import match_equality, tostring

__author__ = "Chris Rose"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class MatchEqualityWrapperTest(unittest.TestCase):
    def testMatcherIsEqualWhenMatchesIsTrue(self):
        matcher = equal_to("bar")
        assert match_equality(matcher) == "bar"

    def testMatcherIsNotEqualWhenMatchesIsFalse(self):
        matcher = equal_to("bar")
        assert match_equality(matcher) != "foo"

    def testMatcherStringIsMatcherDescription(self):
        matcher = equal_to("bar")
        assert str(match_equality(matcher)) == tostring(matcher)

    def testMatcherReprIsMatcher(self):
        matcher = equal_to("bar")
        assert repr(match_equality(matcher)) == tostring(matcher)

    def testMatchesWhenProvidedAnObject(self):
        assert match_equality("bar") == "bar"


if __name__ == "__main__":
    unittest.main()



import unittest

from hamcrest.library.collection.isin import is_in
from hamcrest_unit_test.matcher_test import MatcherTest

from .sequencemixin import GeneratorForm, SequenceForm

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


sequence = ("a", "b", "c")


class IsInTestBase(object):
    def testReturnsTrueIfArgumentIsInSequence(self):
        matcher = is_in(self._sequence(*sequence))

        self.assert_matches("has a", matcher, "a")
        self.assert_matches("has b", matcher, "b")
        self.assert_matches("has c", matcher, "c")
        self.assert_does_not_match("no d", matcher, "d")

    def testHasReadableDescription(self):
        self.assert_description("one of ('a', 'b', 'c')", is_in(self._sequence(*sequence)))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(is_in(self._sequence(*sequence)), "a")

    def testMismatchDescriptionShowsActualArgument(self):
        self.assert_mismatch_description("was 'bad'", is_in(self._sequence(*sequence)), "bad")

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was 'bad'", is_in(self._sequence(*sequence)), "bad")


class IsInConcreteSequenceTest(MatcherTest, IsInTestBase, SequenceForm):
    pass


class IsInGeneratorTest(MatcherTest, IsInTestBase, GeneratorForm):
    pass


if __name__ == "__main__":
    unittest.main()



import os
import re

from setuptools import find_packages, setup

# need to kill off link if we're in docker builds
if os.environ.get("PYTHON_BUILD_DOCKER", None) == "true":
    del os.link

os.chdir(os.path.dirname(os.path.realpath(__file__)))


def read(fname):
    return open(fname).read()


# On Python 3, we can't "from hamcrest import __version__" (get ImportError),
# so we extract the variable assignment and execute it ourselves.
fh = open("src/hamcrest/__init__.py")

# this will be overridden
__version__ = None
try:
    for line in fh:
        if re.match("__version__.*", line):
            exec(line)

finally:
    if fh:
        fh.close()

assert __version__ is not None

REQUIREMENTS_DOCS = ["sphinx~=3.0", "alabaster~=0.7"]
TESTS_BASIC = [
    "pytest>=5.0",
    "pytest-sugar",
    "pytest-xdist",
    "coverage[toml]",
    # No point on Pypy thanks to https://github.com/python/typed_ast/issues/111
    "pytest-mypy-plugins; platform_python_implementation != 'PyPy'",
    "types-mock",
]
TESTS_NUMPY = ["numpy"]
DEV_TOOLS = [
    "towncrier",
    "twine",
    "pytest-mypy",
    "flake8",
    "black",
    "tox",
    "tox-pyenv",
    "tox-asdf",
]


params = dict(
    name="PyHamcrest",
    version=__version__,  # flake8:noqa
    author="Chris Rose",
    author_email="offline@offby1.net",
    description="Hamcrest framework for matcher objects",
    license="New BSD",
    platforms=["All"],
    keywords="hamcrest matchers pyunit unit test testing unittest unittesting",
    url="https://github.com/hamcrest/PyHamcrest",
    download_url="http://pypi.python.org/packages/source/P/PyHamcrest/PyHamcrest-%s.tar.gz"
    % __version__,
    packages=find_packages("src"),
    package_dir={"": "src"},
    package_data={"hamcrest": ["py.typed"]},
    provides=["hamcrest"],
    long_description=read("README.rst"),
    long_description_content_type="text/x-rst",
    python_requires=">=3.5",
    install_requires=[],
    extras_require={
        "docs": REQUIREMENTS_DOCS,
        "tests": TESTS_BASIC,
        "tests-numpy": TESTS_BASIC + TESTS_NUMPY,
        "dev": REQUIREMENTS_DOCS + TESTS_BASIC + DEV_TOOLS,
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: Jython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development",
        "Topic :: Software Development :: Quality Assurance",
        "Topic :: Software Development :: Testing",
    ],
)

all_params = dict(params.items())
setup(**all_params)



import re
import unittest

import pytest
from hamcrest.core.selfdescribing import SelfDescribing
from hamcrest.core.string_description import StringDescription

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class FakeSelfDescribing(SelfDescribing):
    def describe_to(self, description):
        description.append_text("DESCRIPTION")


class StringDescriptionTest(unittest.TestCase):
    def setUp(self):
        self.description = StringDescription()

    def testLetsSelfDescribingObjectDescribeItself(self):
        self.description.append_description_of(FakeSelfDescribing())
        self.assertEqual("DESCRIPTION", str(self.description))

    def testDescribesStringInQuotes(self):
        self.description.append_description_of("FOO")
        self.assertEqual("'FOO'", str(self.description))

    def testWrapsNonSelfDescribingObjectInAngleBrackets(self):
        self.description.append_description_of(42)
        self.assertEqual("<42>", str(self.description))

    def testShouldNotAddAngleBracketsIfObjectDescriptionAlreadyHasThem(self):
        self.description.append_description_of(object())
        expected = re.compile("<object object at 0x[0-9a-fA-F]+>")
        self.assertTrue(expected.match(str(self.description)))

    def testDescribeUnicodeStringAsUnicode(self):
        self.description.append_description_of("\u05d0")
        self.assertEqual("'\u05d0'", str(self.description))


# below is a set of things that should append without error to string
# descriptions
@pytest.mark.parametrize("valid_input", (b"bytes", "unicode"))
def test_description_append_valid_input(valid_input):
    desc = StringDescription()
    desc.append(valid_input)
    str(desc)


def test_description_append_invalid_input():
    desc = StringDescription()
    desc.append(chr(239))


if __name__ == "__main__":
    unittest.main()



from hamcrest.core.base_matcher import BaseMatcher

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class NeverMatch(BaseMatcher):

    mismatch_description = "NEVERMATCH"

    def matches(self, item, mismatch_description=None):
        if mismatch_description:
            self.describe_mismatch(item, mismatch_description)
        return False

    def describe_mismatch(self, item, mismatch_description):
        mismatch_description.append_text(NeverMatch.mismatch_description)



import unittest

from hamcrest.library.number.iscloseto import Decimal, close_to, isnumeric
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsCloseToTest(MatcherTest):
    def testEvaluatesToTrueIfArgumentIsEqualToAValueWithinSomeError(self):
        matcher = close_to(1.0, 0.5)

        self.assert_matches("equal", matcher, 1.0)
        self.assert_matches("less but within delta", matcher, 0.5)
        self.assert_matches("greater but within delta", matcher, 1.5)

        self.assert_does_not_match("too small", matcher, 0.4)
        self.assert_does_not_match("too large", matcher, 1.6)

    def testMatcherCreationAcceptsOtherNumericTypes(self):
        close_to(int(5), int(1))

    def testMatcherCreationRequiresNumbers(self):
        self.assertRaises(TypeError, close_to, "a", 0.5)
        self.assertRaises(TypeError, close_to, 1.0, "a")

    def testFailsIfMatchingAgainstNonNumber(self):
        self.assert_does_not_match("not a number", close_to(1.0, 0.5), "a")

    def testHasAReadableDescription(self):
        self.assert_description("a numeric value within <0.5> of <1.0>", close_to(1.0, 0.5))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(close_to(1.0, 0.5), 1.0)

    def testMismatchDescriptionShowsActualDeltaIfArgumentIsNumeric(self):
        self.assert_mismatch_description("<1.7> differed by <0.7>", close_to(1.0, 0.5), 1.7)

    def testMismatchDescriptionShowsActualArgumentIfNotNumeric(self):
        self.assert_mismatch_description("was 'bad'", close_to(1.0, 0.5), "bad")

    def testDescribeMismatchShowsActualDeltaIfArgumentIsNumeric(self):
        self.assert_describe_mismatch("<1.7> differed by <0.7>", close_to(1.0, 0.5), 1.7)

    def testDescribeMismatchShowsActualArgumentIfNotNumeric(self):
        self.assert_describe_mismatch("was 'bad'", close_to(1.0, 0.5), "bad")

    def testMatcherSupportsDecimal(self):
        matcher = close_to(Decimal("1.0"), Decimal("0.5"))

        self.assert_matches("equal", matcher, Decimal("1.4"))


try:
    import numpy as np

    NUMPY_AVAILABLE = True
except ImportError:
    NUMPY_AVAILABLE = False


class IsNumericTest(unittest.TestCase):
    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_int(self):
        self.assertTrue(isnumeric(np.int(1)), "Platform integer (normally either int32 or int64)")

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_int8(self):
        self.assertTrue(isnumeric(np.int8(1)), "Byte (-128 to 127)")

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_int16(self):
        self.assertTrue(isnumeric(np.int16(1)), "Integer (-32768 to 32767)")

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_int32(self):
        self.assertTrue(isnumeric(np.int32(1)), "Integer (-2147483648 to 2147483647)")

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_int64(self):
        self.assertTrue(
            isnumeric(np.int64(1)), "Integer (9223372036854775808 to 9223372036854775807)"
        )

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_uint8(self):
        self.assertTrue(isnumeric(np.uint8(1)), "Unsigned integer (0 to 255)")

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_uint16(self):
        self.assertTrue(isnumeric(np.uint16(1)), "Unsigned integer (0 to 65535)")

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_uint32(self):
        self.assertTrue(isnumeric(np.uint32(1)), "Unsigned integer (0 to 4294967295)")

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_uint64(self):
        self.assertTrue(isnumeric(np.uint64(1)), "Unsigned integer (0 to 18446744073709551615)")

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_float(self):
        self.assertTrue(isnumeric(np.float(1)), "Shorthand for float64.")

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_float16(self):
        self.assertTrue(
            isnumeric(np.float16(1)),
            "Half precision float: sign bit, 5 bits exponent, 10 bits mantissa",
        )

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_float32(self):
        self.assertTrue(
            isnumeric(np.float32(1)),
            "Single precision float: sign bit, 8 bits exponent, 23 bits mantissa",
        )

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_float64(self):
        self.assertTrue(
            isnumeric(np.float64(1)),
            "Double precision float: sign bit, 11 bits exponent, 52 bits mantissa",
        )

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_complex(self):
        self.assertTrue(isnumeric(np.complex(1)), "Shorthand for complex128.")

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_complex64(self):
        self.assertTrue(
            isnumeric(np.complex64(1)),
            "Complex number, represented by two 32-bit floats (real and imaginary components)",
        )

    @unittest.skipUnless(NUMPY_AVAILABLE, "Skipped because it needs NumPy")
    def test_numpy_numeric_type_complex128(self):
        self.assertTrue(
            isnumeric(np.complex128(1)),
            "Complex number, represented by two 64-bit floats (real and imaginary components)",
        )


if __name__ == "__main__":
    unittest.main()



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest

from hamcrest.core.core.isanything import anything
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsAnythingTest(MatcherTest):
    def testAlwaysEvaluatesToTrue(self):
        self.assert_matches("None", anything(), None)
        self.assert_matches("object", anything(), object())
        self.assert_matches("string", anything(), "hi")

    def testHasUsefulDefaultDescription(self):
        self.assert_description("ANYTHING", anything())

    def testCanOverrideDescription(self):
        description = "DESCRIPTION"
        self.assert_description(description, anything(description))

    def testMatchAlwaysSucceedsSoShouldNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(anything(), "hi")


if __name__ == "__main__":
    unittest.main()



import sys

sys.path.append("..")

from hamcrest.core.base_matcher import BaseMatcher
from hamcrest.core.helpers.hasmethod import hasmethod

from hamcrest import *
import unittest
import datetime


class IsGivenDayOfWeek(BaseMatcher):
    """Matches dates that fall on a given day of the week."""

    def __init__(self, day):
        self.day = day  # Monday is 0, Sunday is 6

    def _matches(self, item):
        """Test whether item matches."""
        if not hasmethod(item, "weekday"):
            return False
        return item.weekday() == self.day

    def describe_to(self, description):
        """Describe the matcher."""
        day_as_string = [
            "Monday",
            "Tuesday",
            "Wednesday",
            "Thursday",
            "Friday",
            "Saturday",
            "Sunday",
        ]
        description.append_text("calendar date falling on ").append_text(day_as_string[self.day])


def on_a_saturday():
    """Factory function to generate Saturday matcher."""
    return IsGivenDayOfWeek(5)


class SampleTest(unittest.TestCase):
    def testDateIsOnASaturday(self):
        """Example of successful match."""
        d = datetime.date(2008, 4, 26)
        assert_that(d, is_(on_a_saturday()))

    def testFailsWithMismatchedDate(self):
        """Example of what happens with date that doesn't match."""
        d = datetime.date(2008, 4, 6)
        assert_that(d, is_(on_a_saturday()))

    def testFailsWithNonDate(self):
        """Example of what happens with object that isn't a date."""
        d = "oops"
        assert_that(d, is_(on_a_saturday()))


if __name__ == "__main__":
    unittest.main()



if __name__ == "__main__":
    import sys

    sys.path.insert(0, "..")
    sys.path.insert(0, "../..")

import unittest

from hamcrest.core.core.anyof import any_of
from hamcrest.core.core.isequal import equal_to
from hamcrest_unit_test.matcher_test import MatcherTest

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class AnyOfTest(MatcherTest):
    def testMatchesIfArgumentSatisfiesEitherOrBothOfTwoOtherMatchers(self):
        self.assert_matches("first matcher", any_of(equal_to("good"), equal_to("bad")), "good")
        self.assert_matches("second matcher", any_of(equal_to("bad"), equal_to("good")), "good")
        self.assert_matches("both matchers", any_of(equal_to("good"), equal_to("good")), "good")

    def testProvidesConvenientShortcutForMatchingWithEqualTo(self):
        self.assert_matches("first matcher", any_of("good", "bad"), "good")
        self.assert_matches("second matcher", any_of("bad", "good"), "good")
        self.assert_matches("both matchers", any_of("good", "good"), "good")

    def testNoMatchIfArgumentFailsToSatisfyEitherOfTwoOtherMatchers(self):
        self.assert_does_not_match(
            "either matcher", any_of(equal_to("bad"), equal_to("bad")), "good"
        )

    def testMatchesIfArgumentSatisfiesAnyOfManyOtherMatchers(self):
        self.assert_matches(
            "matcher in the middle",
            any_of(
                equal_to("bad"), equal_to("bad"), equal_to("good"), equal_to("bad"), equal_to("bad")
            ),
            "good",
        )

    def testNoMatchIfArgumentFailsToSatisfyAnyOfManyOtherMatchers(self):
        self.assert_does_not_match(
            "all matchers",
            any_of(
                equal_to("bad"), equal_to("bad"), equal_to("bad"), equal_to("bad"), equal_to("bad")
            ),
            "good",
        )

    def testHasAReadableDescription(self):
        self.assert_description(
            "('good' or 'bad' or 'ugly')",
            any_of(equal_to("good"), equal_to("bad"), equal_to("ugly")),
        )

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(any_of(equal_to("good"), equal_to("bad")), "good")

    def testMismatchDescriptionDescribesFirstFailingMatch(self):
        self.assert_mismatch_description(
            "was 'ugly'", any_of(equal_to("bad"), equal_to("good")), "ugly"
        )

    def testDescribeMismatch(self):
        self.assert_describe_mismatch(
            "was 'ugly'", any_of(equal_to("bad"), equal_to("good")), "ugly"
        )


if __name__ == "__main__":
    unittest.main()



import unittest

from hamcrest.core.core.isequal import equal_to
from hamcrest.library.collection.issequence_containing import has_item, has_items
from hamcrest_unit_test.matcher_test import MatcherTest

from .quasisequence import QuasiSequence
from .sequencemixin import GeneratorForm, SequenceForm

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class IsSequenceContainingTestBase(object):
    def testMatchesASequenceThatContainsAnElementMatchingTheGivenMatcher(self):
        self.assert_matches(
            "sequence contains 'a'", has_item(equal_to("a")), self._sequence("a", "b", "c")
        )

    def testNoMatchIfSequenceDoesntContainAnElementMatchingTheGivenMatcher(self):
        self.assert_does_not_match(
            "sequence without 'a'", has_item(equal_to("a")), self._sequence("b", "c")
        )
        self.assert_does_not_match("empty", has_item(equal_to("a")), [])

    def testProvidesConvenientShortcutForMatchingWithEqualTo(self):
        self.assert_matches("sequence contains 'a'", has_item("a"), self._sequence("a", "b", "c"))
        self.assert_does_not_match("sequence without 'a'", has_item("a"), self._sequence("b", "c"))

    def testMatchesAnyConformingSequence(self):
        self.assert_matches("quasi-sequence", has_item(1), QuasiSequence())
        self.assert_does_not_match("non-sequence", has_item(1), object())

    def testHasAReadableDescription(self):
        self.assert_description("a sequence containing 'a'", has_item("a"))

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(has_item("a"), self._sequence("a", "b"))

    def testMismatchDescriptionShowsActualArgument(self):
        self.assert_mismatch_description("was <42>", has_item("a"), 42)

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("was <42>", has_item("a"), 42)


class IsConcreteSequenceContaining(MatcherTest, SequenceForm, IsSequenceContainingTestBase):
    pass


class IsGeneratorContaining(MatcherTest, GeneratorForm, IsSequenceContainingTestBase):
    pass


class IsSequenceContainingItemsTestBase(object):
    def testShouldMatchCollectionContainingAllItems(self):
        self.assert_matches(
            "contains all items",
            has_items(equal_to("a"), equal_to("b"), equal_to("c")),
            self._sequence("a", "b", "c"),
        )

    def testProvidesConvenientShortcutForMatchingWithEqualTo(self):
        self.assert_matches(
            "Values automatically wrapped with equal_to",
            has_items("a", "b", "c"),
            self._sequence("a", "b", "c"),
        )

    def testShouldMatchCollectionContainingAllItemsInDifferentOrder(self):
        self.assert_matches(
            "all items in different order", has_items("a", "b", "c"), self._sequence("c", "b", "a")
        )

    def testShouldMatchCollectionContainingAllItemsPlusExtras(self):
        self.assert_matches(
            "all items plus extras",
            has_items("a", "b", "c"),
            self._sequence("e", "c", "b", "a", "d"),
        )

    def testNoMatchIfCollectionDoesntSatisfyAllMatchers(self):
        self.assert_does_not_match(
            "missing 'a'", has_items("a", "b", "c"), self._sequence("e", "c", "b", "d")
        )

    def testHasAReadableDescription(self):
        self.assert_description(
            "(a sequence containing 'a' and a sequence containing 'b')", has_items("a", "b")
        )

    def testSuccessfulMatchDoesNotGenerateMismatchDescription(self):
        self.assert_no_mismatch_description(has_items("a", "b"), self._sequence("a", "b"))

    def testMismatchDescriptionShowsFirstUnmetMatcherAndActualArgument(self):
        self.assert_mismatch_description(
            "a sequence containing 'a' was <42>", has_items("a", "b"), 42
        )

    def testDescribeMismatch(self):
        self.assert_describe_mismatch("a sequence containing 'a' was <42>", has_items("a", "b"), 42)


class IsConcreteSequenceContainingItemsTest(
    MatcherTest, IsSequenceContainingItemsTestBase, SequenceForm
):
    pass


class IsGeneratorSequenceContainingItemsTest(
    MatcherTest, IsSequenceContainingItemsTestBase, GeneratorForm
):
    pass


if __name__ == "__main__":
    unittest.main()



# encoding: utf-8
import unittest
import warnings

from hamcrest.core.assert_that import assert_that
from hamcrest.core.core.isequal import equal_to


def u(x):
    return x


__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"


class AssertThatTest(unittest.TestCase):
    def testShouldBeSilentOnSuccessfulMatch(self):
        assert_that(1, equal_to(1))

    def testAssertionErrorShouldDescribeExpectedAndActual(self):
        expected = "EXPECTED"
        actual = "ACTUAL"

        expectedMessage = "\nExpected: 'EXPECTED'\n     but: was 'ACTUAL'\n"

        with self.assertRaises(AssertionError) as e:
            assert_that(actual, equal_to(expected))

        self.assertEqual(expectedMessage, str(e.exception))

    def testAssertionErrorShouldIncludeOptionalReason(self):
        expected = "EXPECTED"
        actual = "ACTUAL"

        expectedMessage = "REASON\nExpected: 'EXPECTED'\n     but: was 'ACTUAL'\n"

        with self.assertRaises(AssertionError) as e:
            assert_that(actual, equal_to(expected), "REASON")

        self.assertEqual(expectedMessage, str(e.exception))

    def testAssertionUnicodeEncodesProperly(self):
        expected = "EXPECTED"
        actual = u("\xdcnic\N{Latin Small Letter O with diaeresis}de")

        with self.assertRaises(AssertionError):
            assert_that(actual, equal_to(expected), "REASON")

    def testCanTestBoolDirectly(self):
        assert_that(True, "should accept True")

        with self.assertRaises(AssertionError) as e:
            assert_that(False, "FAILURE REASON")
        self.assertEqual("FAILURE REASON", str(e.exception))

    def testCanTestBoolDirectlyWithoutReason(self):
        assert_that(True)

        with self.assertRaises(AssertionError) as e:
            assert_that(False)

        self.assertEqual("Assertion failed", str(e.exception))

    def testWarnsForMatcherAsArg1(self):
        assert_that(True)

        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            assert_that(equal_to(1))

            self.assertEqual(len(w), 1)
            self.assertTrue("arg1 should be boolean" in str(w[-1].message))


if __name__ == "__main__":
    unittest.main()
